/*
Copyright (C) 1999/2000 Hendrik Lipka.
All rights reserved.

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation;
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along with this
program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave,
Cambridge, MA 02139, USA or see http://www.gnu.org

If you encounter problems or have a suggestion, please mail to:
hli@hendriklipka.de
*/
package de.hendriklipka.iwg.data;

import java.util.*;

public class ClassDescriptor
{
    public String _package=null;
    public String[] _imports=new String[0];
    public String _name=null;
    public String[] _interfaces=new String[0];
    public ParameterDescriptor[] _fields=new ParameterDescriptor[0];
    public MethodDescriptor[] _methods=new MethodDescriptor[0];

    protected Hashtable _knownTypes=new Hashtable();

    public void addInterface(String interfaceName)
    {
        String[] sa=new String[_interfaces.length+1];
        System.arraycopy(_interfaces,0,sa,0,_interfaces.length);
        sa[sa.length-1]=interfaceName;
        _interfaces=sa;
        addImport(interfaceName);
    }

    public void addMethod(MethodDescriptor method)
    {
        String sig=method.getSignature();
        for (int i = 0; i < _methods.length; i++)
        {
            MethodDescriptor methodDesc = _methods[i];
            if (methodDesc.getSignature().equals(sig))
            {
                System.err.println("WARNING: found method with duplicate signature while adding\n\t"+method._name+" from "+method._origin+". method will not be added!");
                if (!method._returnType.equals(methodDesc._returnType))
                    System.err.println("ERROR: methods have different return types.\n\tThis will lead to compilation errors!");
                return;
            }
        }
        MethodDescriptor[] ma=new MethodDescriptor[_methods.length+1];
        System.arraycopy(_methods,0,ma,0,_methods.length);
        ma[ma.length-1]=method;
        _methods=ma;
        for (int i = 0; i < method._params.length; i++)
        {
            ParameterDescriptor param = method._params[i];
            addImport(param._type);
        }
        addImport(method._returnType);
    }

    public String addField(String name, String type)
    {
        ParameterDescriptor[] fa=new ParameterDescriptor[_fields.length+1];
        System.arraycopy(_fields,0,fa,0,_fields.length);

        boolean isUnique=true;
        String uName=name;
        int suffix=1;
        do
        {
            isUnique=true;
            for (int i = 0; i < _fields.length; i++)
            {
                ParameterDescriptor field = _fields[i];
                if (field._name.equals(uName))
                {
                    uName=name+suffix;
                    isUnique=false;
                }
            }
        } while (!isUnique);
        fa[fa.length-1]=getPDForType(type,uName);
        _fields=fa;
        return uName;
    }

    protected ParameterDescriptor getPDForType(String type, String name)
    {
        ParameterDescriptor pd=new ParameterDescriptor();
        pd._name=name;
        pd._type=type;
        addImport(type);
        return pd;
    }

    protected void addImport(String className)
    {
        try
        {
            Class c=Class.forName(className);
            if (c.isArray())
                return;
            if (c.isPrimitive())
                return;
        }
        catch (ClassNotFoundException e)
        {
            return;
        }
        Object o=_knownTypes.get(className);
        if (null==o)
        {
            _knownTypes.put(className,className);
            String[] sa=new String[_imports.length+1];
            System.arraycopy(_imports,0,sa,0,_imports.length);
            sa[sa.length-1]=className;
            _imports=sa;
        }
    }

    public void shortenTypeNames()
    {
        Arrays.sort(_imports);
        Arrays.sort(_methods);
        Arrays.sort(_fields);
        Hashtable shortNamesMap=new Hashtable();
        Hashtable knownShortNames=new Hashtable();
        Hashtable dupNames=new Hashtable();
        Enumeration e=_knownTypes.keys();
        while (e.hasMoreElements())
        {
            String type = (String) e.nextElement();
            String shortType=type;
            int pos=type.lastIndexOf('.');
            if (-1!=pos)
                shortType=type.substring(pos+1);
            Object o=knownShortNames.get(shortType);
            if (null==o)
            {
                knownShortNames.put(shortType,type);
                shortNamesMap.put(type,shortType);
            }
            else
            {
                dupNames.put(type,shortType);
                dupNames.put(o,shortType);
                shortNamesMap.put(type,type);
                shortNamesMap.put(o,o);
            }
        }
        for (int i = 0; i < _interfaces.length; i++)
        {
            String anInterface = _interfaces[i];
            String newName=(String)shortNamesMap.get(anInterface);
            if (null!=newName)
            {
                _interfaces[i]=newName;
            }
        }
        for (int i = 0; i < _fields.length; i++)
        {
            ParameterDescriptor field = _fields[i];
            String newName=(String)shortNamesMap.get(field._type);
            if (null!=newName)
            {
                field._type=newName;
            }
        }
        for (int i = 0; i < _methods.length; i++)
        {
            MethodDescriptor method = _methods[i];
            for (int j = 0; j < method._params.length; j++)
            {
                ParameterDescriptor param = method._params[j];
                String newName=(String)shortNamesMap.get(param._type);
                if (null!=newName)
                {
                    param._type=newName;
                }
            }
            String retType=method._returnType;
            String newName=(String)shortNamesMap.get(retType);
            if (null!=newName)
            {
                method._returnType=newName;
            }
        }
    }
}
