/*
Copyright (C) 1999/2000 Hendrik Lipka.
All rights reserved.

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation;
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along with this
program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave,
Cambridge, MA 02139, USA or see http://www.gnu.org

If you encounter problems or have a suggestion, please mail to:
hli@hendriklipka.de
*/
package de.hendriklipka.iwg.standalone;

import de.hendriklipka.iwg.IJavaCreator;
import de.hendriklipka.iwg.JavaCreatorBase;
import de.hendriklipka.iwg.IBodyGenerator;
import de.hendriklipka.iwg.data.ClassDescriptor;
import de.hendriklipka.iwg.data.ParameterDescriptor;
import de.hendriklipka.iwg.data.MethodDescriptor;

import java.util.*;
import java.io.*;

public class JavaFileCreator extends JavaCreatorBase
{
    protected IBodyGenerator _bodyGen=null;
    protected String _sourcePath=null;

    public JavaFileCreator(String sourcePath,IBodyGenerator bodyGen)
    {
        super();
        _sourcePath=sourcePath;
        _bodyGen=bodyGen;
    }

    public void create(ClassDescriptor classDesc)
    {
        String fileName=classDesc._package+File.separator+classDesc._name;
        fileName=fileName.replace('.',File.separatorChar);
        fileName=_sourcePath+File.separator+fileName;
        fileName+=".java";
        File f=new File(fileName);
        File dir=f.getParentFile();
        dir.mkdirs();
        PrintWriter writer=null;
        try
        {
            System.out.println("creating file "+fileName);
            writer=new PrintWriter(new FileWriter(fileName));
            writeHeader(classDesc,writer);
            writer.println();
            writeImports(classDesc,writer);
            writer.println();
            writeClassDefinition(classDesc, writer);
            writeFields(classDesc, writer);
            writer.println();
            writeConstructor(classDesc,writer);
            writer.println();
            writeMethods(classDesc,writer);
            writeClassFinish(classDesc,writer);
            writeFooter(classDesc,writer);
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }
        finally
        {
            if (null!=writer)
            {
                writer.close();
            }
            System.out.println("finished");
        }
    }

    protected void writeHeader(ClassDescriptor classDesc, PrintWriter writer)
    {
        String[] sa=_bodyGen.createHeader();
        writeStringArray(sa,writer,"");
        if (null!=classDesc._package)
            writer.println("package "+classDesc._package+";");
    }

    protected void writeImports(ClassDescriptor classDesc, PrintWriter writer)
    {
        for (int i = 0; i < classDesc._imports.length; i++)
        {
            String anImport = classDesc._imports[i];
            writer.println("import "+anImport+";");
        }
    }

    private void writeClassDefinition(ClassDescriptor classDesc, PrintWriter writer)
    {
        writer.print("public class "+classDesc._name+" implements ");
        for (int i = 0; i < classDesc._interfaces.length; i++)
        {
            String anInterface = classDesc._interfaces[i];
            writer.print(anInterface);
            if (i!=(classDesc._interfaces.length-1))
                writer.print(", ");
        }
        writer.println();
        writer.println("{");
    }

    private void writeFields(ClassDescriptor classDesc, PrintWriter writer)
    {
        for (int i = 0; i < classDesc._fields.length; i++)
        {
            ParameterDescriptor field = classDesc._fields[i];
            writer.println(_indent+"private "+field._type+" "+field._name+"=null;");
        }
    }

    protected void writeConstructor(ClassDescriptor classDesc, PrintWriter writer)
    {
        String[] sa=_bodyGen.generateConstructorInterface(classDesc);
        writeStringArray(sa,writer,_indent);
        writer.println(_indent+"{");
        sa=_bodyGen.generateConstructorBody(classDesc);
        writeStringArray(sa,writer,_2indent);
        writer.println(_indent+"}");
    }
    
    protected void writeMethods(ClassDescriptor classDesc, PrintWriter writer)
    {
        for (int i = 0; i < classDesc._methods.length; i++)
        {
            MethodDescriptor method = classDesc._methods[i];
            writeMethod(method,writer);
            if (i!=(classDesc._methods.length-1))
                writer.println();
        }
    }

    protected void writeMethod(MethodDescriptor method, PrintWriter writer)
    {
        String[] sa=_bodyGen.generateMethodInterface(method);
        writeStringArray(sa,writer,_indent);
        writer.println(_indent+"{");
        sa=_bodyGen.generateMethodBody(method);
        writeStringArray(sa,writer,_2indent);
        writer.println(_indent+"}");
    }

    protected void writeClassFinish(ClassDescriptor classDesc, PrintWriter writer)
    {
        writer.println("}");
    }

    protected void writeFooter(ClassDescriptor classDesc, PrintWriter writer)
    {
        String[] sa=_bodyGen.createFooter();
        writeStringArray(sa, writer,"");
    }

    private void writeStringArray(String[] sa, PrintWriter writer, String indent)
    {
        for (int i = 0; i < sa.length; i++)
        {
            String s = sa[i];
            writer.println(indent+s);
        }
    }
}